#!/usr/bin/env python

"""
dynamips_properties_dialog.py
Copyright (C) 2006  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk
import gobject
from gui_router import dynamips
from dynamips_lib import Dynamips

class DynamipsPropertiesDialog(object):
    slot_num = 7

    def __init__(self, parent, router=None):
        self.parent = parent
        self.router = router
        self.__build_gui()
        #self.__fill_default()
        # run the dialog
        ret = self.w.run()
        self.w.hide()

    def __build_gui(self):
        self.w = gtk.Dialog("Dynamips Manager", self.parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)

        #self.w.set_size_request(300, 300)
        # the main content
        #self.w.connect("destroy", lambda w: gtk.main_quit())
        table_layout = gtk.VBox() #Table(rows=2, columns=2, homogeneous=False)
        #table_layout.set_row_spacings(5)
        #table_layout.set_col_spacings(5)
        #print table_layout.get_homogeneous()

        # create a TreeStore with one string column to use as the model
        self.liststore = gtk.ListStore(str, str)

        # we'll add some data now - 4 rows with 3 child rows each
        for d in dynamips:
            self.liststore.append([d.host, d.port])

        # create the TreeView using liststore
        self.treeview = gtk.TreeView(self.liststore)
        self.scroll = gtk.ScrolledWindow()
        self.scroll.add(self.treeview)

        # create the TreeViewColumn to display the data
        self.tvcolumn = gtk.TreeViewColumn('host')
        self.tvcolumn2 = gtk.TreeViewColumn('port')

        # add tvcolumn to treeview
        self.treeview.append_column(self.tvcolumn)
        self.treeview.append_column(self.tvcolumn2)

        # create a CellRendererText to render the data
        self.cell = gtk.CellRendererText()
        self.cell1 = gtk.CellRendererText()

        # add the cell to the tvcolumn and allow it to expand
        self.tvcolumn.pack_start(self.cell, True)
        self.tvcolumn2.pack_start(self.cell1, True)

        # set the cell "text" attribute to column 0 - retrieve text
        # from that column in liststore
        self.tvcolumn.add_attribute(self.cell, 'text', 0)
        self.tvcolumn2.add_attribute(self.cell1, 'text', 1)

        # make it searchable
        self.treeview.set_search_column(0)

        # Allow sorting on the column
        self.tvcolumn.set_sort_column_id(0)

        # Allow drag and drop reordering of rows
        self.treeview.set_reorderable(True)

        #button_del = gtk.Button(stock=gtk.STOCK_REMOVE)
        #button_del.connect("clicked", self.del_host, gtk.RESPONSE_OK)

        # the new dynamips entry
        label_host = gtk.Label('Host: ')
        self.entry_host = gtk.Entry()
        self.entry_host.set_max_length(50)
        self.entry_host.set_width_chars(20)
        label_port = gtk.Label('Port: ')
        self.entry_port = gtk.Entry()
        self.entry_port.set_max_length(5)
        self.entry_port.set_width_chars(5)
        label_wdir = gtk.Label('Working dir: ')
        self.entry_wdir = gtk.Entry()
        self.entry_wdir.set_max_length(100)
        self.entry_wdir.set_width_chars(35)
        button_add = gtk.Button(stock=gtk.STOCK_ADD)
        button_add.connect("clicked", self.add_host, gtk.RESPONSE_OK)
        hb = gtk.HBox()
        hb.pack_start(label_host)
        hb.pack_start(self.entry_host)
        hb.pack_start(label_port)
        hb.pack_start(self.entry_port)
        wb = gtk.HBox()
        wb.pack_start(label_wdir)
        wb.pack_start(self.entry_wdir)
        wb.pack_start(button_add)

        if len(dynamips)==0:
            try:
                d = Dynamips(host='localhost', port=7200)
                self.entry_host.set_text('localhost')
                self.entry_port.set_text('7200')
            except:
                pass
        ypos = 0
        b = gtk.HBox()
        b.pack_start(self.scroll)
        b2 = gtk.VBox()
        #b2.pack_end(button_del, expand=False)
        b.pack_start(b2, expand=False, fill=False)

        table_layout.pack_start(b)
        table_layout.pack_start(hb, expand=False)
        table_layout.pack_start(wb, expand=False)

        self.w.vbox.pack_start(table_layout)
        # the button panel
        button_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        button_ok = gtk.Button(stock=gtk.STOCK_OK)
        self.w.action_area.pack_start(button_cancel)
        self.w.action_area.pack_start(button_ok)
        button_cancel.connect("clicked", self.clicked, gtk.RESPONSE_CANCEL)
        button_ok.connect("clicked", self.clicked, gtk.RESPONSE_OK)

        if self.entry_host.get_text()=='localhost':
            self.entry_wdir.grab_focus()
        else:
            self.entry_host.grab_focus()

        #self.w.add(self.treeview)
        self.w.show_all()


    def add_host(self, widget, data):
        host = self.entry_host.get_text()
        try:
            port = int(self.entry_port.get_text())
        except:
            return
        self.liststore.append([host, port])
        try:
            d = Dynamips(host=host, port=port)
            if self.entry_wdir.get_text()!='':
                d.workingdir = self.entry_wdir.get_text()
            d.reset()      # reset the server
            dynamips.append(d)
        except: # FIXME cannot connect to server...
            pass


    def del_host(self, widget, data):
        pass


    def clicked(self, widget, data):
        if data==gtk.RESPONSE_APPLY or data==gtk.RESPONSE_OK:
            self.w.destroy()
        else:
            #FIXME delete any changes?
            self.w.destroy()
