#!/usr/bin/env python

"""
ethsw_properties_dialog.py
Copyright (C) 2007  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk
import gobject
from gui_router import dynamips
from gui_ethsw import GuiEthSwitch
from dynamips_properties_dialog import DynamipsPropertiesDialog

class EthSwitchPropertiesDialog:
    def __init__(self, parent, ethswitch=None):
        self.parent = parent
        self.ethswitch = ethswitch
        self.w = gtk.Dialog("Ethernet Switch properties", parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)

        self.__build_gui()
        self.__fill_default()
        # run the dialog
        self.w.run()
        self.w.hide()


    def __build_gui(self):
        #by default, create 8 ports
        if self.ethswitch is not None:
            self.num_ports = max(8, len(self.ethswitch.ports)+2)
        else:
            self.num_ports = 8

        self.mapping_layout = gtk.Table(rows=self.num_ports+4, columns=3)
        self.mapping_layout.set_row_spacings(5)
        self.mapping_layout.set_col_spacings(5)
        table_layout = gtk.Table(rows=4, columns=2)
        table_layout.set_row_spacings(5)
        table_layout.set_col_spacings(5)

        # the dynamips entry
        label_dynamips = gtk.Label('Dynamips: ')
        self.list_dynamips = gtk.ListStore(str)
        for d in dynamips:
            self.list_dynamips.append(["%s:%d" % (d.host, d.port)])
        self.combo_dynamips = gtk.ComboBox(self.list_dynamips)
        cell = gtk.CellRendererText()
        self.combo_dynamips.pack_start(cell, True)
        self.combo_dynamips.add_attribute(cell, 'text', 0)
        button_edit_dynamips = gtk.Button(stock=gtk.STOCK_ADD)
        button_edit_dynamips.connect("clicked", self.edit_dynamips, gtk.RESPONSE_OK)

        # the name entry
        label_name = gtk.Label('Name: ')
        self.entry_name = gtk.Entry()
        self.entry_name.set_max_length(15)
        self.entry_name.set_width_chars(8)

        self.entry_vlan = [None] * self.num_ports
        self.check_trunking = [None] * self.num_ports
        for i in range(self.num_ports):
            # define the entry box for the vlan
            self.entry_vlan[i] = gtk.Entry()
            self.entry_vlan[i].set_max_length(3)
            self.entry_vlan[i].set_width_chars(5)
            # define the dot1Q check Box
            self.check_trunking[i] = gtk.CheckButton("dot1Q")

        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        for i in range(self.num_ports):
            self.mapping_layout.attach(gtk.Label("Port %d, VLAN: " % i),
                                       0, 1, i, i+1)
            self.mapping_layout.attach(self.entry_vlan[i], 1, 2, i, i+1,
                            xoptions=gtk.EXPAND, yoptions=gtk.EXPAND)
            self.mapping_layout.attach(self.check_trunking[i], 2, 3, i, i+1,
                            xoptions=gtk.EXPAND, yoptions=gtk.EXPAND)


        ypos = 0
        label_dynamips.set_alignment(0, 0.5)
        table_dyn = gtk.Table(rows=1, columns=2)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.combo_dynamips)
        al1 = gtk.Alignment()
        al1.set(0.1, 0.0, 0.0, 1.0)
        al1.add(button_edit_dynamips)
        table_dyn.attach(al, 0, 1, 0, 1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)
        table_dyn.attach(al1, 1, 2, 0, 1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)
        table_layout.attach(label_dynamips, 0, 1, ypos, ypos+1)
        table_layout.attach(table_dyn, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_name.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.entry_name)
        al.show()
        table_layout.attach(label_name, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        table_layout.attach(self.mapping_layout, 0, 4, ypos, ypos+1)

        #self.button_addmapping = gtk.Button('Add a mapping')
        #self.button_addmapping.connect("clicked", self.addmapping, 'add')
        #ypos += 1
        #table_layout.attach(self.button_addmapping, 1, 3, ypos, ypos+1)

        label_color = gtk.Label('Color: ')
        ypos += 1
        table_layout.attach(label_color, 0, 1, ypos, ypos+1)
        self.button_color = gtk.ColorButton()
        table_layout.attach(self.button_color, 1, 2, ypos, ypos+1)

        self.w.vbox.pack_start(table_layout)
        # the button panel
        button_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        button_apply = gtk.Button(stock=gtk.STOCK_APPLY)
        button_ok = gtk.Button(stock=gtk.STOCK_OK)
        self.w.action_area.pack_start(button_cancel)
        self.w.action_area.pack_start(button_apply)
        self.w.action_area.pack_start(button_ok)
        button_cancel.connect("clicked", self.clicked, gtk.RESPONSE_CANCEL)
        button_apply.connect("clicked", self.clicked, gtk.RESPONSE_APPLY)
        button_ok.connect("clicked", self.clicked, gtk.RESPONSE_OK)
        self.w.show_all()


    def __fill_default(self):
        # put some defaults values
        if self.ethswitch is None:
            if len(dynamips)>0:
                self.combo_dynamips.set_active(0)
            self.button_color.set_color(
                        gtk.gdk.Color(int(0.8*65535),
                                      int(0.2*65535),
                                      int(0.2*65535)))
        else:
            for i,d in enumerate(dynamips):
                if d.host==self.ethswitch.dynamips.host \
                    and d.port==self.ethswitch.dynamips.port:
                    self.combo_dynamips.set_active(i)
            self.entry_name.set_text(self.ethswitch.name)
            self.button_color.set_color(self.ethswitch.gdkcolor)
            for i, l in enumerate(self.ethswitch.ports):
                if not (l == [None, None]):
                    vlan, dot1Q = l
                    self.entry_vlan[i].set_text(str(vlan))
                    self.check_trunking[i].set_active(dot1Q==True)
            for i, l in enumerate(self.ethswitch.unusedports):
                if not (l == [None, None]):
                    vlan, dot1Q = l
                    self.entry_vlan[i].set_text(str(vlan))
                    self.check_trunking[i].set_active(dot1Q==True)

    def edit_dynamips(self, widget, data):
        DynamipsPropertiesDialog(self.parent, None)
        for d in dynamips:
            self.list_dynamips.append(["%s:%d" % (d.host, d.port)])
        self.combo_dynamips.set_active(0)



    def addmapping(self, widget, data=None):
        self.mapping_layout.resize(len(self.combo_port)+1, 4)
        i = len(self.combo_port)
        cell = gtk.CellRendererText()
        self.combo_port.append([])
        self.combo_port[i].append(gtk.ComboBox(self.list_store))
        self.combo_port[i][0].pack_start(cell, True)
        self.combo_port[i][0].add_attribute(cell, 'text', 0)
        self.combo_port[i][0].set_active(0)     #FIXME or preselected
        self.combo_port[i][0].show()
        self.combo_port[i].append(gtk.ComboBox(self.list_store))
        self.combo_port[i][1].pack_start(cell, True)
        self.combo_port[i][1].add_attribute(cell, 'text', 0)
        self.combo_port[i][1].set_active(0)     #FIXME or preselected
        self.combo_port[i][1].show()
        self.entry_dlci.append([])
        # the leftmost dlci entry
        self.entry_dlci[i].append(gtk.Entry())
        self.entry_dlci[i][0].set_max_length(3)
        self.entry_dlci[i][0].set_width_chars(5)
        self.entry_dlci[i][0].show()
        # the rightmost dlci entry
        self.entry_dlci[i].append(gtk.Entry())
        self.entry_dlci[i][1].set_max_length(3)
        self.entry_dlci[i][1].set_width_chars(5)
        self.entry_dlci[i][1].show()
        self.mapping_layout.attach(self.entry_dlci[i][0], 0, 1, i, i+1)
        self.mapping_layout.attach(self.combo_port[i][0], 1, 2, i, i+1,
                        xoptions=gtk.EXPAND, yoptions=gtk.EXPAND)
        self.mapping_layout.attach(self.combo_port[i][1], 2, 3, i, i+1,
                        xoptions=gtk.EXPAND, yoptions=gtk.EXPAND)
        self.mapping_layout.attach(self.entry_dlci[i][1], 3, 4, i, i+1)

    def d_error(self, msg):
        """ Display an error message"""
        dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                            gtk.BUTTONS_OK, msg)
        dialog.run()
        dialog.destroy()


    def close(self, widget, event):
        w.response(0)


    def clicked(self, widget, data):
        if data==gtk.RESPONSE_CANCEL:
            self.w.destroy()
        elif data==gtk.RESPONSE_APPLY or gtk.RESPONSE_OK:
            if self.ethswitch is None:
                dynamips_text = self.combo_dynamips.get_active_text()
                if dynamips_text is None:
                    self.d_error("Please select a dynamips instance")
                    return
                name = self.entry_name.get_text()
                if name is None or name == "":
                    self.d_error("Please select a frsw name")
                    return
                for d in dynamips:
                    if ("%s:%d" % (d.host, d.port))==dynamips_text:
                        self.ethswitch = GuiEthSwitch(dynamips=d, name=name)
                        break

            c = self.button_color.get_color()
            self.ethswitch.set_bgcolor(c.red/65535.0, c.green/65535.0,
                            c.blue/65535.0)
            for i in range(self.num_ports):
                vlan = self.entry_vlan[i].get_text()
                dot1Q = self.check_trunking[i].get_active()
                try:
                    int(vlan)
                    if vlan!=' ':
                        self.ethswitch.addport(i, int(vlan), dot1Q)
                except:
                    pass

        if data==gtk.RESPONSE_OK:
            self.w.destroy()
