#!/usr/bin/env python

"""
gui_frsw.py
Copyright (C) 2006  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


from dynamips_lib import *
import gtk
from gui_router import GuiRouter

class GuiATMSwitch(ATMSW):

    def __init__(self, x=10, y=10, dynamips=None, name=None):
        ATMSW.__init__(self, dynamips=dynamips, name=name)

        self.ports = [None] * 8 # 8 ports switch to start with?

        self.started = False
        self.suspended = False

        self.px = self.x = x
        self.py = self.y = y
        self.dx = self.dy = 0
        self.name_delta = 0

        self.connections = []
        self.mappings = []

        self.w, self.h = 4+len(self.ports)*14, 32
        self.colorbg_r, self.colorbg_g, self.colorbg_b = 0.49, 0.73, 0.96


    def init_draw(self, context):
        xbearing, ybearing, width, height, xadvance, yadvance = \
                context.text_extents(self.name)
        self.name_delta = width/2


    def which_port(self, x, y):
        """Returns the port number at coordinates x,y (if any)"""
        if y<self.y+2 or y>self.y+14:
            return None
        for port in range(len(self.ports)):
            if (x-self.x)>(2+port*14) and (x-self.x)<(14+port*14):
                return port
        return None

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        return (self.x + 8 + port*14, self.y + 8)

    def addconnection(self, connection, side):
        # this port
        port = self.which_port(connection.xa, connection.ya)
        if side==0:
            if port is not None:
                # remote pa
                deviceb = connection.deviceb
                if isinstance(deviceb, GuiRouter):
                    pa_slotb, xlb, ylb = connection.deviceb.which_pa(
                                            connection.xb, connection.yb)
                    remote_pa = connection.deviceb.slot[pa_slotb]
                    remote_port = remote_pa.which_port(xlb, ylb)
                    # connect the ports
                    self.connect(port, self.dynamips, remote_pa, remote_port)
                    self.ports[port] = [connection, side]
                else:
                    print self, '-->', deviceb, 'no connect'
            else:
                return
        self.connections.append([port, connection, side])

    def addmapping(self, porta, vpia, portb, vpib):
        for maps in self.mappings:
            if [porta, vpia, portb, vpib]==maps:
                # mapping already exists, we don't add it again
                print 'mapping already exists'
                return
        self.mappings.append([porta, vpia, portb, vpib])
        if ':' in vpia and ':' in vpib:
            vpia, vcia = vpia.split(':')
            vpib, vcib = vpib.split(':')
            self.mapvc(porta, int(vpia), int(vcia), portb, int(vpib), int(vcib))
            self.mapvc(portb, int(vpib), int(vcib), porta, int(vpia), int(vcia))
        else:
            self.mapvp(porta, vpia, portb, vpib)
            self.mapvp(portb, vpib, porta, vpia)

    def inside(self, x, y):
        """ returns true if x, y is inside the device """
        return x>self.x and x<self.x+self.w and y>self.y and y<self.y+self.h


    def inside_pa_port(self, x, y):
        """ returns true if x, y is inside one of the switch's port"""
        if not self.inside(x, y):
            return False
        if y>self.y+2 and y<self.y+14:
            x -= self.x
            for i in range(len(self.ports)):
                if x>(2+i*14) and x<(14+i*14):
                    return True
        return False

    def inside_empty_port(self, x, y):
        """ returns true if x, y is inside one of the switch's port that is empty"""
        if not self.inside(x, y):
            return False
        if y>self.y+2 and y<self.y+14:
            x -= self.x
            for i in range(len(self.ports)):
                if x>(2+i*14) and x<(14+i*14):
                    if self.ports[i] is None:
                        return True
                    else:
                        return False
        return False


    def setdelta(self, x, y):
        """ the x,y delta to which the pointer clicked inside the device """
        self.dx = x - self.x
        self.dy = y - self.y

    def moveto(self, x, y):
        """ Move the device to position x, y,
        preserving old positions in vars for drawing """
        tx = x - self.x - self.dx
        ty = y - self.y - self.dy
        for p,c,e in self.connections:
            if e == 0:
                c.translate0to(tx, ty)
            else:
                c.translate1to(tx, ty)
        self.px = self.x #old position
        self.py = self.y
        self.x = x - self.dx
        self.y = y - self.dy

    def draw(self, context):
        """Draw the swith on a canvas"""
        context.set_source_rgb(self.colorbg_r, self.colorbg_g, self.colorbg_b)
        context.rectangle(self.x, self.y, len(self.ports)*14+2, 32)
        context.fill()
        context.set_line_width(1)
        # draw each port
        context.set_source_rgb(0, 0, 0)
        for i, p in enumerate(self.ports):
            context.rectangle(self.x + 2 + i*14 + 0.5, self.y + 2 + 0.5, 12, 12)
        # draw frame
        context.rectangle(self.x + 0.5, self.y +0.5, 2+len(self.ports)*14, 16)
        context.rectangle(self.x+ 0.5, self.y+16 +0.5, 2+len(self.ports)*14, 16)
        context.stroke()
        # draw name
        context.move_to(self.x + 60 - self.name_delta, self.y + 30)
        context.show_text(self.name)

    def dirty_area(self):
        """ Returns a (x0, y0, x1, y1) dirty area """
        minx = miny = 60000
        maxx = maxy = 0
        for p,c,e in self.connections:
            minx = min(minx, c.xa, c.xb)
            maxx = max(maxx, c.xa, c.xb)
            miny = min(miny, c.ya, c.yb)
            maxy = max(maxy, c.ya, c.yb)
        minx = min(minx, self.x, self.px)
        maxx = max(maxx, self.x + self.w, self.px + self.w)
        miny = min(miny, self.y, self.py)
        maxy = max(maxy, self.y + self.h, self.py + self.h)
        return (minx, miny, maxx, maxy+12)

    def intersect(self, area):
        left, top, right, bottom = area
        l, t, r, b = self.x, self.y, self.x+self.w, self.y+self.h
        if left < l: left = l
        if top < t: top = t
        if right > r: right = r
        if bottom > b: bottom = b
        return not (left >= right or top >= bottom)

    def set_bgcolor(self, r, g, b):
        self.colorbg_r, self.colorbg_g, self.colorbg_b = r, g, b

    def _get_gdkcolor(self):
        return gtk.gdk.Color(int(self.colorbg_r*65535),
                             int(self.colorbg_g*65535),
                             int(self.colorbg_b*65535))

    gdkcolor = property(_get_gdkcolor, doc='return the gtk.gdk background color')

