#!/usr/bin/env python

"""
router_properties_dialog.py
Copyright (C) 2006, 2007 Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import os
from lab import lab
from gui_router import GuiRouter

class RouterPropertiesDialog(object):

    def update_idlepc(self, widget, event):
        """Update the idlepc value accordingly to the selected IOS.

        The method first looks into already configured routers to apply the same
        idle_pc value, failing that it will read the dynagenidledb.ini file
        to find a idle_pc value for the selected IOS.
        """
        if self.entry_idlepc.get_text()!='':
            return # do not update existing value
        file = self.button_ios.get_filename()
        if file is None:
            return
        for d in lab.devices:
            if isinstance(d, GuiRouter) and d.image==file:
                if d.idlepc is not None:
                    self.entry_idlepc.set_text(str(d.idlepc))
        # if still None, look for it in the dynagenidledb.ini file
        sel_ios = file[file.rfind(os.path.sep)+1:]
        if self.entry_idlepc.get_text() in [None, '']:
            fname = os.path.expanduser('~' + os.path.sep + 'dynagenidledb.ini')
            try:
                fin = open(fname)
                for line in fin.readlines():
                    ios, idle_val = line.split('=')
                    ios = ios.strip()
                    idle_val = idle_val.strip()
                    if ios==sel_ios:
                        idle_val = idle_val.replace('"', '')
                        self.entry_idlepc.set_text(str(idle_val))
                fin.close()
            except IOError, e:
                if e.errno == 2: # no dynagenidledb file
                    pass
                else:
                    print e


